using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace PostProcessing
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        GridVertexPositionColorTexture grid;
        Texture2D background;
        BasicEffect basicEffect;
        Vector2 pinchCenter = new Vector2(240, 800);

        public Game1()
        {
            
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
            graphics.IsFullScreen = true;
            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            grid = new GridVertexPositionColorTexture();
            grid.BuildGeometry(48, 80, new Vector2(10, 10));

            background = Content.Load<Texture2D>("background");
            basicEffect = new BasicEffect(GraphicsDevice);
            basicEffect.TextureEnabled = true;
            basicEffect.Projection = Matrix.CreateOrthographicOffCenter(0, 480, 800, 0, 0, 10);
            basicEffect.View = basicEffect.World = Matrix.Identity;

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            grid.ResetUVs();

            //use sine wave for intensity
            float intensity = (float)Math.Sin(gameTime.TotalGameTime.TotalSeconds) * 100;

            TouchCollection collection = TouchPanel.GetState();
            foreach (TouchLocation point in collection)
            {
                if (point.State == TouchLocationState.Moved)
                {
                    //apply pinch at all touch locations
                    ApplyPinch(grid, point.Position, intensity);
                }
            }
            
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            GraphicsDevice.SamplerStates[0] = SamplerState.LinearClamp;
            basicEffect.Texture = background;
            basicEffect.CurrentTechnique.Passes[0].Apply();
            //draw the grid using the basicEffect
            grid.Draw(GraphicsDevice);
            
            
            base.Draw(gameTime);
        }


        public void ApplyPinch(GridVertexPositionColorTexture grid, Vector2 center, float intensity)
        {
            Vector2 gridSize = new Vector2(grid.Width, grid.Height) * grid.CellSize;

            for (int i = 0; i < grid.Vertices.Length; i++)
            {
                //Get the position in pixels
                Vector2 pos = grid.GetUV0(i) * gridSize;

                //Get the direction from the current grid vertex to the center-of-pinch
                Vector2 dir = pos - center;
                float length = dir.Length();
                float minDisplacement = -length;
                if (dir.Length() != 0)
                {
                    dir.Normalize();
                }

                //Compute the displacement, but not lower than the minDisplacement
                Vector2 displacement = dir * Math.Max(intensity, minDisplacement);
                //Find the new position
                Vector2 newPos = pos + displacement;
                //Set the new Texture Coordinates
                grid.SetUV0(i, newPos / gridSize);
            }
        }
    }
}
